package protofmt

import "github.com/emicklei/proto"

// reflector is a Visitor that can tell the short type name of a Visitee.
type reflector struct {
	name string
}

// sole instance of reflector
var namer = new(reflector)

func (r *reflector) VisitMessage(m *proto.Message)         { r.name = "Message" }
func (r *reflector) VisitService(v *proto.Service)         { r.name = "Service" }
func (r *reflector) VisitSyntax(s *proto.Syntax)           { r.name = "Syntax" }
func (r *reflector) VisitPackage(p *proto.Package)         { r.name = "Package" }
func (r *reflector) VisitOption(o *proto.Option)           { r.name = "Option" }
func (r *reflector) VisitImport(i *proto.Import)           { r.name = "Import" }
func (r *reflector) VisitNormalField(i *proto.NormalField) { r.name = "NormalField" }
func (r *reflector) VisitEnumField(i *proto.EnumField)     { r.name = "EnumField" }
func (r *reflector) VisitEnum(e *proto.Enum)               { r.name = "Enum" }
func (r *reflector) VisitComment(e *proto.Comment)         { r.name = "Comment" }
func (r *reflector) VisitOneof(o *proto.Oneof)             { r.name = "Oneof" }
func (r *reflector) VisitOneofField(o *proto.OneOfField)   { r.name = "OneOfField" }
func (r *reflector) VisitReserved(rs *proto.Reserved)      { r.name = "Reserved" }
func (r *reflector) VisitRPC(rpc *proto.RPC)               { r.name = "RPC" }
func (r *reflector) VisitMapField(f *proto.MapField)       { r.name = "NormalField" } // handle as
func (r *reflector) VisitGroup(g *proto.Group)             { r.name = "Group" }
func (r *reflector) VisitExtensions(e *proto.Extensions)   { r.name = "Extensions" }

// nameOfVisitee returns the short type name of a Visitee.
func nameOfVisitee(e proto.Visitee) string {
	e.Accept(namer)
	return namer.name
}
